#include <stdio.h>
#include <stdlib.h>
#include <limits.h>

typedef struct Node {
    struct Node *next;
    int value;
} Node;

typedef struct List {
    Node *head;
    Node *tail;
    int len;
} List;

typedef struct Stack {
    Node *head;
    Node *minStack;  
    int min;
} Stack;

typedef struct Queue {
    Node *head;
} Queue;

void allocateList(List *list) {
    list->head = NULL;
    list->tail = NULL;
    list->len = 0;
}

Node *getNode(Node *head, int pos) {
    int counter = 0;
    while (counter < pos && head) {
        head = head->next;
        counter++;
    }
    return head;
}

Node *getFirst(Node *head) {
    return head;
}

Node *getLast(Node *head) {
    if (head == NULL) return NULL;
    while (head->next) {
        head = head->next;
    }
    return head;
}

int addFirst(List *list, int val) {
    if (list == NULL) return 0;
    Node *newNode = malloc(sizeof(Node));
    newNode->next = list->head;
    newNode->value = val;
    if (list->head == NULL) list->tail = newNode;
    list->head = newNode;
    list->len++;
    return 1;
}

int addLast(List *list, int value) {
    if (list == NULL) return 0;
    Node *newNode = malloc(sizeof(Node));
    newNode->next = NULL;
    newNode->value = value;
    if (list->head == NULL) {
        list->head = newNode;
    } else {
        list->tail->next = newNode;
    }
    list->tail = newNode;
    list->len++;
    return 1;
}

int addAtPosition(List *list, int value, int position) {
    if (!list || list->len < position || position < 0) return 0;
    if (list->len == 0) {
        Node *node = malloc(sizeof(Node));
        list->head = node;
        list->tail = node;
        node->value = value;
        node->next = NULL;
        list->len++;
        return 1;
    }
    if (position == list->len) {
        return addLast(list, value);
    }
    Node *iter = list->head;
    if (position == 0) {
        return addFirst(list, value);
    }
    for (int i = 0; i < position - 1; i++) {
        iter = iter->next;
    }
    Node *newNode = malloc(sizeof(Node));
    newNode->next = iter->next;
    iter->next = newNode;
    newNode->value = value;
    list->len++;
    return 1;
}

int removeFirst(List *list) {
    if (list->head == NULL) {
        return 1;
    }
    Node *first = list->head;
    list->head = first->next;
    free(first);
    list->len--;
    if (list->head == NULL) list->tail = NULL;
    return 1;
}

int removeLast(List *list) {
    if (list->head == NULL) {
        return 0;
    }
    if (list->head->next == NULL) {
        return removeFirst(list);
    }
    Node *iter = list->head;
    while (iter->next->next != NULL) {
        iter = iter->next;
    }
    free(iter->next);
    iter->next = NULL;
    list->tail = iter;
    list->len--;
    return 1;
}

int removeAtPosition(List *list, int position) {
    if (list->head == NULL || position < 0) return 0;
    if (position == 0) {
        return removeFirst(list);
    } else if (position == list->len - 1) {
        return removeLast(list);
    } else {
        Node *previous = getNode(list->head, position - 1);
        Node *next = previous->next->next;
        free(previous->next);
        previous->next = next;
        list->len--;
        return 1;
    }
}

int getListSize(Node *head) {
    int counter = 0;
    while (head != NULL) {
        head = head->next;
        counter++;
    }
    return counter;
}

void clearList(List *list) {
    if (!list) return;
    Node *current = list->head;
    while (current != NULL) {
        Node *next = current->next;
        free(current);
        current = next;
    }
    list->head = NULL;
    list->tail = NULL;
    list->len = 0;
}

void allocateStack(Stack *stack) {
    stack->head = NULL;
    stack->minStack = NULL;
    stack->min = INT_MAX;
}

int push(Stack *stack, int value) {
    if (stack == NULL) return 0;
    Node *newNode = malloc(sizeof(Node));
    newNode->next = stack->head;
    newNode->value = value;
    stack->head = newNode;
    if (value < stack->min) {
        Node *minNode = malloc(sizeof(Node));
        minNode->next = stack->minStack;
        minNode->value = value;
        stack->minStack = minNode;
        stack->min = value;
    }
    return 1;
}

int pop(Stack *stack) {
    if (stack->head == NULL || stack == NULL) {
        return 1;
    }
    Node *first = stack->head;
    if (first->value == stack->min) {
        Node *firstMin = stack->minStack;
        stack->minStack = firstMin->next;
        stack->min = (stack->minStack != NULL) ? stack->minStack->value : INT_MAX;
        free(firstMin);
    }
    stack->head = first->next;
    free(first);
    return 1;
}

int getStackSize(Stack *stack) {
    return getListSize(stack->head);
}

void freeStack(Stack *stack) {
    if (stack == NULL) return;
    Node *current = stack->head;
    while (current != NULL) {
        Node *next = current->next;
        free(current);
        current = next;
    }
    stack->head = NULL;
    stack->minStack = NULL;
}

int getMin(Stack *stack) {
    return stack->min;
}

void allocateQueue(Queue *queue) {
    queue->head = NULL;
}

int enqueue(Queue *queue, int value) {
    if (queue == NULL) return -1;
    Node *newNode = malloc(sizeof(Node));
    newNode->next = NULL;
    newNode->value = value;
    if (queue->head == NULL) {
        queue->head = newNode;
    } else {
        Node *last = getLast(queue->head);
        last->next = newNode;
    }
    return 1;
}

int dequeue(Queue *queue) {
    if (queue == NULL) return -1;
    if (queue->head == NULL) return 1;
    Node *deleteNode = queue->head;
    queue->head = deleteNode->next;
    free(deleteNode);
    return 1;
}

int getQueueSize(Queue *queue) {
    return getListSize(queue->head);
}

void freeQueue(Queue *queue) {
    if (queue == NULL) return;
    Node *current = queue->head;
    while (current != NULL) {
        Node *next = current->next;
        free(current);
        current = next;
    }
    queue->head = NULL;
}
