#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

static const char LINE_SEPARATOR[] = "\n\n===============\n\n";

int **allocateMatrix(int n, int m);

void freeMatrix(int **matrix, int n);

void scanfMatrix(int **matrix, int n, int m);

void printfMatrix(int **matrix, int n, int m);

int **copyMatrix(int **matrix, int n, int m);

void transposeMatrix(int **matrix, int n, int m);

int main() {
    int N;
    int M;
    scanf("%d %d", &N, &M);

    int **matrix = allocateMatrix(N, M);
    scanfMatrix(matrix, N, M);
    printfMatrix(matrix, N, M);
    printf(LINE_SEPARATOR);

    int **matrixCopy = copyMatrix(matrix, N, M);
    freeMatrix(matrix, N);
    printfMatrix(matrixCopy, N, M);
    printf(LINE_SEPARATOR);

    transposeMatrix(matrixCopy, N, M);
    printfMatrix(matrixCopy, N, M);
    freeMatrix(matrixCopy, N);

    return EXIT_SUCCESS;
}

int **allocateMatrix(int n, int m) {
    int **matrix = malloc(n * sizeof(int *));
    assert(matrix != NULL);
    for (int i = 0; i < n; i++) {
        matrix[i] = malloc(m * sizeof(int));
        assert(matrix[i] != NULL);
    }
    return matrix;
}

void freeMatrix(int **matrix, int n) {
    for (int i = 0; i < n; i++) {
        free(matrix[i]);
    }
    free(matrix);
}

void scanfMatrix(int **matrix, int n, int m) {
    for(int i = 0; i < n; i++) {
        for(int j = 0; j < m; j++) {
            scanf("%d", &matrix[i][j]);
        }
    }
}

void printfMatrix(int **matrix, int n, int m) {
    for(int i = 0; i < n; i++) {
        for(int j = 0; j < m; j++) {
            printf("%d", matrix[i][j]);
        }
        printf("\n");
    }
}

int **copyMatrix(int **matrix, int n, int m) {
    int **copyMat = allocateMatrix(n, m);
    for(int i = 0; i < n; i++) {
        for(int j = 0; j < m; j++) {
            copyMat[i][j] = matrix[i][j];
        }
    }
    return copyMat;
}

int transposeMatrix(int ***matrix, int *n, int *m){
    if (matrix == NULL || *matrix == NULL || *n <= 0 || *m <= 0) {
        return 0;
    }
    int **transposed = malloc(*m * sizeof(int *));
    if (transposed == NULL) {
        return 0;
    }
    for (int i = 0; i < *m; i++) {
        transposed[i] = malloc(*n * sizeof(int));
        if (transposed[i] == NULL) {
            for (int j = 0; j < i; j++) {
                free(transposed[j]);
            }
            free(transposed);
            return 0;
        }
    }
    for (int i = 0; i < *n; i++) {
        for (int j = 0; j < *m; j++) {
            transposed[j][i] = (*matrix)[i][j];
        }
    }
    for (int i = 0; i < *n; i++) {
        free((*matrix)[i]);
    }
    free(*matrix);
    *matrix = transposed;
    int temp = *n;
    *n = *m;
    *m = temp;
    return 1;
}
