#define SIZE 26 

typedef struct TrieNode {
    struct TrieNode *children[SIZE];
    bool word;
} TrieNode;

TrieNode* createTrieNode() {
    TrieNode* new = (TrieNode*)malloc(sizeof(TrieNode));
    new->word = false;
    for (int i = 0; i < SIZE; i++) {
        new->children[i] = NULL;
    }
    return new;
}

typedef struct {
    TrieNode* root;
} MagicDictionary;

MagicDictionary* magicDictionaryCreate() {
    MagicDictionary* obj = (MagicDictionary*)malloc(sizeof(MagicDictionary));
    obj->root = createTrieNode();
    return obj;
}

void insertWord(TrieNode* root, char* word) {
    TrieNode* curr = root;
    int len = strlen(word);
    for (int i = 0; i < len; i++) {
        int j = word[i] - 'a';
        if (curr->children[j] == NULL) {
            curr->children[j] = createTrieNode();
        }
        curr = curr->children[j];
    }
    curr->word = true;
}

void magicDictionaryBuildDict(MagicDictionary* obj, char** dictionary, int dictionarySize) {
    for (int i = 0; i < dictionarySize; i++) {
        insertWord(obj->root, dictionary[i]);
    }
}

bool searchWord(TrieNode* root, char* word, int index, int count) {
    while (root != NULL && index < strlen(word)) {
        char c = word[index];
        for (int i = 0; i < SIZE; i++) {
            if(root->children[i] != NULL) {
                if(i != c - 'a' && count == 0) {
                    if (searchWord(root->children[i], word, index + 1, count + 1)) {
                        return true;
                    }
                } else if(i == c - 'a') {
                    if (searchWord(root->children[i], word, index + 1, count)) {
                        return true;
                    }
                }
            }
        }
        return false;
    }
    return root != NULL && root->word && count == 1 && index >= strlen(word);
}

bool magicDictionarySearch(MagicDictionary* obj, char* word) {
    return searchWord(obj->root, word, 0, 0);
}

void freeTrie(TrieNode* node) {
    if (node == NULL) {
        return;
    }
    for (int i = 0; i < SIZE; i++) {
        freeTrie(node->children[i]);
    }
    free(node);
}

void magicDictionaryFree(MagicDictionary* obj) {
    freeTrie(obj->root);
    free(obj);
}
