#include <stdio.h>
#include <pthread.h>
#include <unistd.h>
#include <stdatomic.h>


typedef struct {
    atomic_int lock;
    atomic_int tid;
} mutex_t;

void mutex_init(mutex_t* mutex) {
    atomic_init(&mutex->lock, 0);
    atomic_init(&mutex->tid, -1);
}

void mutex_lock(mutex_t* mutex) {
    int tid = get_thread_id();
    while (1) {
        int expected = 0;
        if (atomic_compare_exchange_weak(&mutex->lock, &expected, 1)) {
            atomic_store(&mutex->tid, tid);
            break;
        }
        usleep(1);
    }
}

void mutex_unlock(mutex_t* mutex) {
    int tid = get_thread_id();
    if (atomic_load(&mutex->tid) == tid) { 
        atomic_store(&mutex->lock, 0); 
        atomic_store(&mutex->tid, -1);
    }
}

mutex_t my_mutex;

void* thread_function(void* arg) {
    mutex_lock(&my_mutex);
    mutex_unlock(&my_mutex);
    return NULL;
}
