#include <stdlib.h>
#include <stdio.h>
#include <string.h>

void swap(void *a, void *b, size_t size) {
    void *temp = malloc(size);
    if (temp == NULL) {
        return;
    }
    memcpy(temp, a, size);
    memcpy(a, b, size);
    memcpy(b, temp, size);
    free(temp);
}

int partition(void *arr, int left, int right, size_t size, int (*compare)(const void *, const void *)) {
    void *pivot = (char *)arr + right * size;
    int i = left - 1;

    for (int j = left; j < right; j++) {
        if (compare((char *)arr + j * size, pivot) <= 0) {
            i++;
            swap((char *)arr + i * size, (char *)arr + j * size, size);
        }
    }
    swap((char *)arr + (i + 1) * size, (char *)arr + right * size, size);
    return i + 1;
}

void Qsort(void *arr, int left, int right, size_t size, int (*compare)(const void *, const void *)) {
    if (left < right) {
        int pi = partition(arr, left, right, size, compare);
        Qsort(arr, left, pi - 1, size, compare);
        Qsort(arr, pi, right, size, compare);
    }
}
void qsort(void *ptr, size_t count, size_t size, int (*comp)(const void *, const void *)) {
    Qsort(ptr, 0, count - 1, size, comp);
}

int compareInt(const void *a, const void *b) {
    return (*(int *)a - *(int *)b);
}

int main() {
    int arrLen;
    if (scanf("%d", &arrLen) == 1) {
        int *arr = malloc(arrLen * sizeof(int));
        if (arr != NULL && arrLen != 0) {
            for (int i = 0; i < arrLen; i++) {
                if (scanf("%d", &arr[i]) != 1) {
                    return 0;
                }
            }
            qsort(arr, arrLen, sizeof(int), compareInt);
            for (int i = 0; i < arrLen; i++) {
                printf("%d ", arr[i]);
            }
        }
        free(arr);
    }
    return 0;
}
